
#if !defined(_NativeInterpreterMacros__)
#define _NativeInterpreterMacros__

#include "NativeInterpreterTypes.h"
#include "NativeInterpreterVm.h"

// read macros
#define READ_OPCODE(op, stream)				\
	do { op = *((uint16_t*)stream); stream += sizeof(uint16_t); } while (false)
#define READ_REGISTER_INDEX(v, stream)		\
	do { v= *((uint8_t*)stream); stream += sizeof(uint8_t); } while (false)
#define READ_IMMI8(imm8, stream)		\
	do { imm8 = *((uint8_t*)stream); stream += sizeof(uint8_t); } while (false)
#define READ_IMMI16(imm16, stream)		\
	do { imm16 = *((uint16_t*)stream); stream += sizeof(uint16_t); } while (false)
#define READ_IMMI32(imm32, stream)		\
	do { imm32 = *((uint32_t*)stream); stream += sizeof(uint32_t); } while (false)
#define READ_IMMI64(imm64, stream)		\
	do { imm64 = *((uint64_t*)stream); stream += sizeof(uint64_t); } while (false)
#define READ_IMMP(immp, stream)		\
	do { immp = *((uintptr_t*)stream); stream += sizeof(uintptr_t); } while (false)
#define READ_IMMF32(immf32, stream)		\
	do { immf32 = *((float*)stream); stream += sizeof(float); } while (false)
#define READ_IMMF64(immf64, stream)		\
	do { immf64 = *((double*)stream); stream += sizeof(double); } while (false)
#define READ_JUMP_OFFSET(offset, stream)		\
	do { offset = *((ptrdiff_t*)stream); stream += sizeof(ptrdiff_t); } while (false)

// write macros
#define WRITE_OPCODE(op, stream, msg, ...)															\
	do {																							\
		stream -= sizeof(uint16_t); *((uint16_t*)stream) = static_cast<uint16_t>(op);				\
		verbose_only(																				\
			char buffer[256];																		\
			VMPI_snprintf(buffer, 256, msg, __VA_ARGS__);											\
			asm_output("%s %s", nanojit::interpreter::getInterpreterOpcodeString(op), buffer);		\
		)																							\
	} while (false)
#define WRITE_REGISTER_INDEX(v, stream)		\
	do { stream -= sizeof(uint8_t); *((uint8_t*)stream) = static_cast<uint8_t>(nanojit::REGNUM(v)); } while (false)
#define WRITE_IMMI8(imm8, stream)		\
	do { stream -= sizeof(uint8_t); *((uint8_t*)stream) = static_cast<uint8_t>(imm8); } while (false)
#define WRITE_IMMI16(imm16, stream)		\
	do { stream -= sizeof(uint16_t); *((uint16_t*)stream) = static_cast<uint16_t>(imm16); } while (false)
#define WRITE_IMMI32(imm32, stream)		\
	do { stream -= sizeof(uint32_t); *((uint32_t*)stream) = imm32; } while (false)
#define WRITE_IMMI64(imm64, stream)		\
	do { stream -= sizeof(uint64_t); *((uint64_t*)stream) = imm64; } while (false)
#define WRITE_IMMP(immp, stream)		\
	do { stream -= sizeof(uintptr_t); *((uintptr_t*)stream) = reinterpret_cast<uintptr_t>(immp); } while (false)
#define WRITE_IMMF32(immf, stream)		\
	do { stream -= sizeof(float); *((float*)stream) = immf; } while (false)
#define WRITE_IMMF64(immd, stream)		\
	do { stream -= sizeof(double); *((double*)stream) = immd; } while (false)
#define WRITE_JUMP_OFFSET(offset, stream)		\
	do { stream -= sizeof(ptrdiff_t); *((ptrdiff_t*)stream) = (offset); } while (false)

#endif // !defined(_NativeInterpreterMacros__)
